/***************************************************************************
**                                                                        **
**  QCustomPlot, an easy to use, modern plotting widget for Qt            **
**  Copyright (C) 2011, 2012, 2013, 2014 Emanuel Eichhammer               **
**                                                                        **
**  This program is free software: you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation, either version 3 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
**  GNU General Public License for more details.                          **
**                                                                        **
**  You should have received a copy of the GNU General Public License     **
**  along with this program.  If not, see http://www.gnu.org/licenses/.   **
**                                                                        **
****************************************************************************
**           Author: Emanuel Eichhammer                                   **
**  Website/Contact: http://www.qcustomplot.com/                          **
**             Date: 27.12.14                                             **
**          Version: 1.3.0                                                **
****************************************************************************/

/************************************************************************************************************
**                                                                                                         **
**  This is the example code for QCustomPlot.                                                              **
**                                                                                                         **
**  It demonstrates basic and some advanced capabilities of the widget. The interesting code is inside     **
**  the "setup(...)Demo" functions of MainWindow.                                                          **
**                                                                                                         **
**  In order to see a demo in action, call the respective "setup(...)Demo" function inside the             **
**  MainWindow constructor. Alternatively you may call setupDemo(i) where i is the index of the demo       **
**  you want (for those, see MainWindow constructor comments). All other functions here are merely a       **
**  way to easily create screenshots of all demos for the website. I.e. a timer is set to successively     **
**  setup all the demos and make a screenshot of the window area and save it in the ./screenshots          **
**  directory.                                                                                             **
**                                                                                                         **
*************************************************************************************************************/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QTimer>
#include "qcustomplot.h" // the header file of QCustomPlot. Don't forget to add it to your project, if you use an IDE, so it gets compiled.
#include "filtre.h"

namespace Ui {
class MainWindow;
}

struct styleTrace
{
    QColor couleur;
    QCPGraph::LineStyle style_ligne;
    QCPScatterStyle::ScatterShape style_point;
};

class MainWindow : public QMainWindow
{
  Q_OBJECT
  
public:
  explicit MainWindow(int nbAxes = 1,QWidget *parent = 0);
    ~MainWindow();
   void  plot(int numAxes, double * ptr_x, double* ptr_y, uint nb_pts, styleTrace paramGraphe);
   void  plot(int numAxe, DataATracer data, styleTrace paramGraphe);
   void  plot(DataATracer data, styleTrace paramGraphe);
   void  plot(int numAxe, DataATracer data);
   void  plot(DataATracer data);
   void  setLegend(int numAxes, const QString &x, const QString &y);
   void  setLegend(const QString &x, const QString &y);
   void  setLogScale(int numAxes, bool log);
   void  setLogScale(bool log);
   void  setGrid(int numAxes, bool grid);
   void  setGrid(bool grid);
   void  setInterpolation(bool interp);
   void  setNomGraphe(const QString &Nom);


private slots:
   void showPointToolTip(QMouseEvent *event);

private:
  Ui::MainWindow *ui;
  QString demoName;
  QCPItemTracer *itemDemoPhaseTracer;
  int currentDemoIndex;
};

#endif // MAINWINDOW_H
