
/* unicodes alphabet grec à partir de 0x03B1
 * α β γ δ ε ζ η θ ι κ λ μ ν ξ ο π ρ ς σ τ υ φ χ ψ ω
 */

#include "filtre.h"
#include "mainwindow.h"
#include <math.h>

void code_tp3_I(void)
{
    // Créez votre filtre ici
    const int NB_COEFF_B=16;
    double coeffB[NB_COEFF_B];
    const int NB_COEFF_A=1;
    double coeffA[NB_COEFF_A]={1};

    // RIF : calcul des coefficents par invariance indicielle comme en TD !
    coeffB[0] = 0;
    for (int i=1; i<NB_COEFF_B; i++)
        coeffB[i] = exp(-(i-1)/5.0) - exp(-i/5.0);

    // RII : coefficients obtenus par transformée bilinéaire
    const int NB_COEFF_B2=2;
    double coeffB2[NB_COEFF_B2]={1.0/11, 1.0/11};
    const int NB_COEFF_A2=2;
    double coeffA2[NB_COEFF_A2]={1, -9.0/11};

    sFiltre filtre_invariance;
    sFiltre filtre_bilineaire;

    initFiltre(&filtre_invariance,NB_COEFF_B,coeffB,NB_COEFF_A,coeffA);
    initFiltre(&filtre_bilineaire,NB_COEFF_B2,coeffB2,NB_COEFF_A2,coeffA2);

    // Declarez vos signaux ici
    const int NB_ECH=20;
    double k[NB_ECH];
    double impulsion[NB_ECH];
    double echelon[NB_ECH];
    for(int i=0; i < NB_ECH; i++)
    {
        if (i==0)
            impulsion[i] = 1;
        else
            impulsion[i] = 0;
        echelon[i] = 1;
        k[i]=i;
    }

    // Filtrez votre signal
    double hk_i[NB_ECH],hk_b[NB_ECH];
    double sindk_i[NB_ECH],sindk_b[NB_ECH];

    calculReponseFiltre(&filtre_invariance,impulsion,hk_i,NB_ECH);
    calculReponseFiltre(&filtre_invariance,echelon,sindk_i,NB_ECH);
    calculReponseFiltre(&filtre_bilineaire,impulsion,hk_b,NB_ECH);
    calculReponseFiltre(&filtre_bilineaire,echelon,sindk_b,NB_ECH);

    // Définissez les données à tracer
    DataATracer data1h,data1s;
    DataATracer data2h,data2s;

    initDataATracer( &data1h ,NB_ECH,k,hk_i);
    initDataATracer( &data2h ,NB_ECH,k,hk_b);
    initDataATracer( &data1s ,NB_ECH,k,sindk_i);
    initDataATracer( &data2s ,NB_ECH,k,sindk_b);

    // Créez une ou plusieurs fenêtres
    MainWindow *w1 = new MainWindow(2);
    MainWindow *w2 = new MainWindow(2);

    styleTrace style1 = {Qt::red,QCPGraph::lsImpulse,QCPScatterStyle::ssCross}, style2 = {Qt::blue,QCPGraph::lsImpulse,QCPScatterStyle::ssPlus};

    w1->plot(0,data1h,style1);
    w1->plot(1,data1s,style1);
    w1->setLegend(0,"k","hk");
    w1->setLegend(1,"k","sindk");
    w1->setWindowTitle("réponses temporelles - synthese par invariance indicielle");
    w1->show();

    w2->plot(0,data2h,style2);
    w2->plot(1,data2s,style2);
    w2->setLegend(0,"k","hk");
    w2->setLegend(1,"k","sindk");
    w2->setWindowTitle("réponses temporelles - synthese par transformee bilinéaire");
    w2->show();

    // Définissez vos fréquences
    const double FECH=5000;
    const double FMIN=0.1;
    const double FMAX=(FECH/2.0);
    const int NB_FREQ=1000;

    double freqLog[NB_FREQ];
    double moduleDb_i[NB_FREQ],moduleDb_b[NB_FREQ];

    for (int ind=0; ind < NB_FREQ; ind++)
    {
        freqLog[ind] = FMIN * pow(pow(FMAX/FMIN,1.0/NB_FREQ),ind);
    }
	
    // Calculez module/phase
    calculModule(&filtre_invariance,freqLog,NB_FREQ,FECH, true,moduleDb_i);
    calculModule(&filtre_bilineaire,freqLog,NB_FREQ,FECH, true,moduleDb_b);

    // ///////// Filtre analogique !!!
    // Définition du filtre modèle : ici les coefficients représentent le numérateur et le dénominateur donnés selon les puissances croissantes de p
        #define RC 0.001
        #define NB_COEFF_NUM 1
        double coeffNum[NB_COEFF_NUM] = {1};
        #define NB_COEFF_DEN 2
        double coeffDen[NB_COEFF_DEN]={1, RC};

        sFiltre filtreModele;
        initFiltre(&filtreModele,NB_COEFF_NUM,coeffNum,NB_COEFF_DEN,coeffDen);

        double moduleModele[NB_FREQ];
        calculModuleAnalogique(&filtreModele,freqLog,NB_FREQ,true,moduleModele);


    // Définissez les données à tracer
    DataATracer data3,data4,data5;

    initDataATracer( &data3 ,NB_FREQ,freqLog,moduleDb_i);
    initDataATracer( &data4 ,NB_FREQ,freqLog,moduleDb_b);
    initDataATracer( &data5 ,NB_FREQ,freqLog,moduleModele);

    // Créez une ou plusieurs fenêtres
    MainWindow *w3 = new MainWindow;

    styleTrace style3 = {Qt::red,QCPGraph::lsLine,QCPScatterStyle::ssNone}, style4 = {Qt::blue,QCPGraph::lsLine,QCPScatterStyle::ssNone},
               style5 = {Qt::black,QCPGraph::lsLine,QCPScatterStyle::ssNone};


    // Tracez vos courbes sur la fenêtre concernée
        w3->setLogScale(true);
        w3->plot(data3,style3);
        w3->plot(data4,style4);
        w3->plot(data5,style5);
        w3->setWindowTitle("tracé des modules");
        w3->setLegend("f","|H|dB");
        w3->setGrid(true);

        w3->show();

}


