
/* unicodes alphabet grec Ã  partir de 0x03B1
 * Î± Î² Î³ Î´ Îµ Î¶ Î· Î¸ Î¹ Îº Î» Î¼ Î½ Î¾ Î¿ Ï€ Ï Ï‚ Ïƒ Ï„ Ï… Ï† Ï‡ Ïˆ Ï‰
 */

#include "filtre.h"
#include "util.h"
#include "mainwindow.h"
#include <cmath>

//Définition des constantes

const double FECH=48000;
const double FMIN=20;
const double FMAX=(FECH/2.0);
const int NB_FREQ=1000;
const double F0 = 632 ;
const double F01 = 6320 ;
const double F02 = 63.2 ;
const double H0_6dB = pow(10,(-6.0/20.0))  ;
const double H0_12dB = pow(10,(12.0/20.0))  ;
const double H0_m12dB = pow(10,(-12.0/20.0)) ;
const double w0 = 2* M_PI *  F0 ;
const double w01 = 2* M_PI *  F01 ;
const double w02 = 2* M_PI *  F02 ;
const double Tech = 1.0/FECH ;
const double Q = 0.7 ;

const int NB_COEFF_B=3;
const int NB_COEFF_A=3;

double moduleModele[NB_FREQ];
double freqLog[NB_FREQ];
double freqLin[NB_FREQ];

double moduleLin [NB_FREQ];

double module_PB_12dB[NB_FREQ] , module_PB_m12dB[NB_FREQ], module_PB_6dB[NB_FREQ] , module_3bde[NB_FREQ];
double phase_PB_F0[NB_FREQ] , phase_PB_F01[NB_FREQ] , phase_PB_F02[NB_FREQ] ;


//---------------------------------------------------------------------------
//                                  Procedures outils
//---------------------------------------------------------------------------

void calculerY_tab_sinus_sweep(const double *ptr_tab_sin1,const double *ptr_tab_sin2, const double *ptr_tab_sin3, double *ptr_tab_sin_tot ,double nb_ech1,double nb_ech2, double  nb_ech3,double  nb_echTot)
{

    for(int i = 0 ; i < nb_ech1 ; i++)
    {
       ptr_tab_sin_tot[i] =  ptr_tab_sin1[i] ;
    }

    for(int i = nb_ech1 ; i < (nb_ech1+nb_ech2) ; i++)
    {
        ptr_tab_sin_tot[i] =  ptr_tab_sin2[i-(int)nb_ech1] ;

    }
    for(int i = (nb_ech1+nb_ech2) ; i < nb_echTot ; i++)
    {
        ptr_tab_sin_tot[i] =  ptr_tab_sin3[i-(int)(nb_ech1+nb_ech2)] ;
    }


}

void calculerY_tab_sinus(const int fsin , double *ptr_tab_sin, double nb_ech)
{
    //Declarations locales
    double t;
    //Actions

    for(int i = 0 ; i < nb_ech ; i++ )
    {
    t =  ((double) i )/FECH ;

        ptr_tab_sin[i] = sin((2*M_PI)* fsin * t);
    }
}

void calculerX_tab(double nb_ech , double *ptr_tab_x)
{
    for(int i = 0 ; i < nb_ech ; i++ )
    ptr_tab_x[i]=i ;
}

void tracer_module(DataATracer data1 , DataATracer data2 , DataATracer data3 ,DataATracer data4 , styleTrace style1 , styleTrace style2 , styleTrace style3, styleTrace style4 )
{
    MainWindow *w_module_filtre = new MainWindow;

        w_module_filtre ->setLogScale(true);
        w_module_filtre ->plot(data1,style1);
        w_module_filtre ->plot(data2,style2);
        w_module_filtre ->plot(data3,style3);
        w_module_filtre ->plot(data4,style4);

        w_module_filtre ->setWindowTitle("tracé des modules des filtres");
        w_module_filtre ->setLegend("f","|H|dB");
        w_module_filtre ->setGrid(true);

        w_module_filtre ->show();
}

void tracer_phase(DataATracer data1 , DataATracer data2 , DataATracer data3 , styleTrace style1 , styleTrace style2 , styleTrace style3  )
{
    MainWindow *w_phase_filtre = new MainWindow;

        w_phase_filtre ->setLogScale(true);
        w_phase_filtre ->plot(data1,style1);
        w_phase_filtre ->plot(data2,style2);
        w_phase_filtre ->plot(data3,style3);

        w_phase_filtre ->setWindowTitle("tracé des phases des filtres");
        w_phase_filtre ->setLegend("f","ɸ(rad)");
        w_phase_filtre ->setGrid(true);

        w_phase_filtre ->show();
}

void tracer_signal(DataATracer data1 , DataATracer data2 , MainWindow *ptr_window , char title[] , styleTrace style1 , styleTrace style2)
{
ptr_window->setLogScale(false);
ptr_window->plot(data1,style1);
ptr_window->plot(data2,style2);
ptr_window->setWindowTitle(title);
ptr_window->setLegend("k","A");
ptr_window->setGrid(true);

ptr_window->show();

}

//----Initialisation des coefficients pour les filtres
void generateEqualizer_center(sFiltre *pFiltre , double H0 , double *ptr_tab_coeffA , double *ptr_tab_coeffB )
    {

    if (H0 > 1)
    {
    calcTransfBilin((1/(w0*w0)) ,(1/(w0*Q)), 1 , (1/(w0*w0)) , (H0/(w0*Q)), 1 , Tech , ptr_tab_coeffB , ptr_tab_coeffA);
    initFiltre(pFiltre,NB_COEFF_B, ptr_tab_coeffB,NB_COEFF_A,ptr_tab_coeffA);
    }
    else
    {
        calcTransfBilin((1/(w0*w0)) ,(1/(H0*w0*Q)), 1 , (1/(w0*w0)) , (1/(w0*Q)), 1 , Tech , ptr_tab_coeffB, ptr_tab_coeffA);
        initFiltre(pFiltre,NB_COEFF_B, ptr_tab_coeffB,NB_COEFF_A,ptr_tab_coeffB);

    }
    }

void generateEqualizer_hf(sFiltre *pFiltre , double H0 , double *ptr_tab_coeffA , double *ptr_tab_coeffB  )
    {
    if (H0 > 1)
    {
    calcTransfBilin((1/(w01*w01)) ,(1/(w01*Q)), 1 , (1/(w01*w01)) , (H0/(w01*Q)), 1 , Tech ,ptr_tab_coeffB, ptr_tab_coeffA);
    initFiltre(pFiltre,NB_COEFF_B, ptr_tab_coeffB,NB_COEFF_A,ptr_tab_coeffA);
    }
    else
    {
        calcTransfBilin((1/(w01*w01)) ,(1/(H0*w01*Q)), 1 , (1/(w01*w01)) , (1/(w01*Q)), 1 , Tech , ptr_tab_coeffB, ptr_tab_coeffA);
        initFiltre(pFiltre,NB_COEFF_B, ptr_tab_coeffB,NB_COEFF_A,ptr_tab_coeffA);

    }
    }

void generateEqualizer_bf(sFiltre *pFiltre , double H0 , double *ptr_tab_coeffA , double *ptr_tab_coeffB)
    {

    if (H0 > 1)
    {
    calcTransfBilin((1/(w02*w02)) ,(1/(w02*Q)), 1 , (1/(w02*w02)) , (H0/(w02*Q)), 1 , Tech , ptr_tab_coeffB, ptr_tab_coeffA);
    initFiltre(pFiltre,NB_COEFF_B, ptr_tab_coeffA,NB_COEFF_A,ptr_tab_coeffA);
    }
    else
    {
        calcTransfBilin((1/(w02*w02)) ,(1/(H0*w02*Q)), 1 , (1/(w02*w02)) , (1/(w02*Q)), 1 , Tech , ptr_tab_coeffB, ptr_tab_coeffA);
        initFiltre(pFiltre,NB_COEFF_B, ptr_tab_coeffB,NB_COEFF_A,ptr_tab_coeffA);

    }
    }

void egaliseur3Bandes(double *pEk , double *pSk , const int nbEch , sFiltre filtre_PB_F02_m6dB, sFiltre filtre_PB_F0_12dB , sFiltre filtre_PB_F01_m12dB )
{
    //Declarations locales
    double Ik1[nbEch] , Ik2[nbEch] ;
    //Actions

    calculReponseFiltre (&filtre_PB_F02_m6dB,pEk, Ik1, nbEch);
    calculReponseFiltre (&filtre_PB_F0_12dB,Ik1, Ik2, nbEch);
    calculReponseFiltre (&filtre_PB_F01_m12dB,Ik2, pSk, nbEch);


}
    //---------------------------------------------------------------------------
    //                                  Procedure principale
    //---------------------------------------------------------------------------
void code_tp4_projet(void)
{
    //---------------------------------------------------------------------------
    //                                  Fabrication des filtres
    //---------------------------------------------------------------------------
    //Declarations locales

    //***Déclaration des styles***
    styleTrace
        style3 = {Qt::red,QCPGraph::lsLine,QCPScatterStyle::ssNone},
        style4 = {Qt::blue,QCPGraph::lsLine,QCPScatterStyle::ssNone},
        style5 = {Qt::black,QCPGraph::lsLine,QCPScatterStyle::ssNone},
        style6 = {Qt::green,QCPGraph::lsLine,QCPScatterStyle::ssNone},
        style7 = {Qt::black,QCPGraph::lsStepCenter,QCPScatterStyle::ssNone},
        style8 = {Qt::red,QCPGraph::lsStepCenter,QCPScatterStyle::ssNone};

    //***Déclarations des Tableaux des coefficients***
    double tab_coeffA_center[NB_COEFF_A] , tab_coeffB_center[NB_COEFF_B];
    double tab_coeffA_bf[NB_COEFF_A] , tab_coeffB_bf[NB_COEFF_B];
    double tab_coeffA_hf[NB_COEFF_A] , tab_coeffB_hf[NB_COEFF_B];


    //***Déclarations des Filtres***
    sFiltre filtre_PB_F02_m6dB, filtre_PB_F0_12dB , filtre_PB_F01_m12dB   ;


    // Definition des échelles logarithmiques et linéaires
    for (int ind=0; ind < NB_FREQ; ind++)
    {
        freqLog[ind] = FMIN * pow(pow(FMAX/FMIN,1.0/NB_FREQ),ind);
        freqLin[ind]=FMIN+ ind* (FMAX-FMIN)/(NB_FREQ-1);
    }

    //Initialisation des filtres
    generateEqualizer_center(&filtre_PB_F0_12dB , H0_12dB , tab_coeffA_center , tab_coeffB_center); //CENTER
    generateEqualizer_bf(&filtre_PB_F02_m6dB , H0_6dB , tab_coeffA_bf , tab_coeffB_bf ); //BF
    generateEqualizer_hf(&filtre_PB_F01_m12dB , H0_m12dB, tab_coeffA_hf, tab_coeffB_hf); //HF

    //Calcul des modules
    calculModule(&filtre_PB_F0_12dB,freqLog,NB_FREQ,FECH, true,module_PB_12dB); //CENTER
    calculModule(&filtre_PB_F02_m6dB,freqLog,NB_FREQ,FECH, true,module_PB_6dB); //BF
    calculModule(&filtre_PB_F01_m12dB,freqLog,NB_FREQ,FECH, true,module_PB_m12dB); //HF

    for(int i = 0 ; i < NB_FREQ; i++)
    {
        module_3bde[i] =   module_PB_12dB[i] + module_PB_m12dB[i]+ module_PB_6dB[i] ;
    }


    //Calcul des phases

    calculPhase(&filtre_PB_F0_12dB,freqLog,NB_FREQ,FECH,phase_PB_F0); //CENTER
    calculPhase(&filtre_PB_F02_m6dB,freqLog,NB_FREQ,FECH,phase_PB_F02); //BF
    calculPhase(&filtre_PB_F01_m12dB,freqLog,NB_FREQ,FECH,phase_PB_F01); //HF

    //Declarations des donnees a tracer
    DataATracer  data_module_PB_bf, data_module_PB_center, data_module_PB_hf , data_module_3bdes; //Data Modules
    DataATracer  data_phase_PB_bf, data_phase_PB_center, data_phase_PB_hf; //Data Phases

    //Initialisation des donnees a tracer
    //***Modules***
    initDataATracer(&data_module_PB_center ,NB_FREQ,freqLog, module_PB_12dB);  //CENTER
    initDataATracer(&data_module_PB_bf ,NB_FREQ,freqLog, module_PB_6dB); //BF
    initDataATracer(&data_module_PB_hf ,NB_FREQ,freqLog, module_PB_m12dB); //HF
    initDataATracer(&data_module_3bdes ,NB_FREQ,freqLog, module_3bde); //3 bandes
    //***Phases***
    initDataATracer(&data_phase_PB_center,NB_FREQ,freqLog, phase_PB_F0);  //CENTER
    initDataATracer(&data_phase_PB_bf ,NB_FREQ,freqLog, phase_PB_F02); //BF
    initDataATracer(&data_phase_PB_hf ,NB_FREQ,freqLog, phase_PB_F01); //HF

    //Tracer des donnees
    tracer_module(data_module_PB_bf , data_module_PB_center , data_module_PB_hf , data_module_3bdes, style4 , style3 , style6 , style7); //Modules
    tracer_phase(data_phase_PB_bf , data_phase_PB_center , data_phase_PB_hf , style4 , style3 , style6); //Phases

    //---------------------------------------------------------------------------
    //                                  Fabrication des signaux
    //---------------------------------------------------------------------------
    //Generation de signaux sinusoidaux
    //***Definition du nombre d'echantillons a calculer***

    #define NB_PERIODES 10 //nombre de périodes du signal a calculer
    #define F_SIN1 20
    #define F_SIN2 632
    #define F_SIN3 20000

    #define NB_ECH_SIN_20hz (const)(1.0/F_SIN1*NB_PERIODES*FECH)
    #define NB_ECH_SIN_632hz (const)(1.0/F_SIN2*NB_PERIODES*FECH)
    #define NB_ECH_SIN_20khz (const)(1.0/F_SIN3*NB_PERIODES*FECH)
    #define NB_ECH_SIN_SWEEP (NB_ECH_SIN_20hz + NB_ECH_SIN_632hz + NB_ECH_SIN_20khz)

    //***Declaration des tableaux des signaux***
    //Axe X
    double posX_20Hz[NB_ECH_SIN_20hz] ;
    double posX_632Hz[NB_ECH_SIN_632hz] ;
    double posX_20kHz[NB_ECH_SIN_20khz] ;
    double posX_sweep[NB_ECH_SIN_SWEEP] ;

    //Axe Y
    double sinusY_20Hz[NB_ECH_SIN_20hz] ;
    double sinusY_632Hz[NB_ECH_SIN_632hz] ;
    double sinusY_20kHz[NB_ECH_SIN_20khz] ;
    double sinusY_sweep[NB_ECH_SIN_SWEEP] ;

    //Calcul des echantillons verticaux (Y)
    calculerY_tab_sinus(F_SIN1 , sinusY_20Hz, NB_ECH_SIN_20hz) ;
    calculerY_tab_sinus(F_SIN2 , sinusY_632Hz, NB_ECH_SIN_632hz) ;
    calculerY_tab_sinus(F_SIN3 , sinusY_20kHz, NB_ECH_SIN_20khz) ;
    calculerY_tab_sinus_sweep(sinusY_20Hz,sinusY_632Hz,sinusY_20kHz, sinusY_sweep , NB_ECH_SIN_20hz, NB_ECH_SIN_632hz, NB_ECH_SIN_20khz, NB_ECH_SIN_SWEEP);

    //Calcul des echantillons horizontaux (X)
    calculerX_tab(NB_ECH_SIN_20hz , posX_20Hz) ;
    calculerX_tab(NB_ECH_SIN_632hz , posX_632Hz) ;
    calculerX_tab(NB_ECH_SIN_20khz , posX_20kHz) ;
    calculerX_tab(NB_ECH_SIN_SWEEP , posX_sweep) ;

    //Filtrage du signal
    double harmoY_20hz [NB_ECH_SIN_20hz];
    double harmoY_632hz [NB_ECH_SIN_632hz];
    double harmoY_20khz [NB_ECH_SIN_20khz];
    double harmoY_sweep [NB_ECH_SIN_SWEEP];

    calculReponseFiltre (&filtre_PB_F0_12dB,sinusY_20Hz, harmoY_20hz, NB_ECH_SIN_20hz);
    calculReponseFiltre (&filtre_PB_F0_12dB,sinusY_632Hz, harmoY_632hz, NB_ECH_SIN_632hz);
    calculReponseFiltre (&filtre_PB_F0_12dB,sinusY_20kHz, harmoY_20khz, NB_ECH_SIN_20khz);

    egaliseur3Bandes(sinusY_sweep, harmoY_sweep , NB_ECH_SIN_SWEEP , filtre_PB_F02_m6dB, filtre_PB_F0_12dB ,filtre_PB_F01_m12dB );

    //Declarations des donnees a tracer
    DataATracer data_sinus1 , data_sinus2 , data_sinus3 , data_sinus4;
    DataATracer data_sinus_filtre1, data_sinus_filtre2, data_sinus_filtre3 , data_sinus_filtre4;

    //Initialisation des donnees a tracer
    //***Sinus***
    initDataATracer(&data_sinus1, NB_ECH_SIN_20hz , posX_20Hz , sinusY_20Hz);
    initDataATracer(&data_sinus2, NB_ECH_SIN_632hz , posX_632Hz , sinusY_632Hz);
    initDataATracer(&data_sinus3, NB_ECH_SIN_20khz , posX_20kHz , sinusY_20kHz);
    initDataATracer(&data_sinus4, NB_ECH_SIN_SWEEP , posX_sweep , sinusY_sweep);
    //***Sinus Filtré***
    initDataATracer(&data_sinus_filtre1, NB_ECH_SIN_20hz , posX_20Hz , harmoY_20hz);
    initDataATracer(&data_sinus_filtre2, NB_ECH_SIN_632hz , posX_632Hz , harmoY_632hz);
    initDataATracer(&data_sinus_filtre3, NB_ECH_SIN_20khz , posX_20kHz , harmoY_20khz);
    initDataATracer(&data_sinus_filtre4, NB_ECH_SIN_SWEEP , posX_sweep , harmoY_sweep);

    //Declarations des fenetres
    MainWindow *w7 = new MainWindow;
    MainWindow *w8 = new MainWindow;
    MainWindow *w9 = new MainWindow;
    MainWindow *w10 = new MainWindow;

    //Tracer des signaux
    tracer_signal(data_sinus1 , data_sinus_filtre1 , w7 , "filtrage sinus a 20hz" , style3 , style5); //20Hz
    tracer_signal(data_sinus2 , data_sinus_filtre2 , w8, "filtrage sinus a 632hz" , style3 , style5); //632Hz
    tracer_signal(data_sinus3 , data_sinus_filtre3 , w9, "filtrage sinus a 20khz", style8 , style7); //20kHz
    tracer_signal(data_sinus4 , data_sinus_filtre4 , w10, "filtrage sinus sweep", style3 , style5); //sweep

}


